"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from typing import Any, Mapping, Optional, Union


class ValidateConnection(BaseSDK):
    def state(
        self,
        *,
        service_id: str,
        unified_api: str,
        request_body: Optional[
            Union[
                models.VaultValidateConnectionStateRequestBody,
                models.VaultValidateConnectionStateRequestBodyTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.VaultValidateConnectionStateResponse:
        r"""Validate Connection State

        This endpoint validates the current state of a given connection. This will perform different checks based on the connection auth type. For basic and apiKey auth types, the presence of required fields is checked.
        For connectors that implement OAuth2, this operation forces the refresh flow for an access token regardless of its expiry.

        Note:
        - Do not include any credentials in the request body. This operation does not persist changes, but only triggers the validation of connection state.
        - If a refresh token flow was performed and successful, the new access token will then be used for subsequent API requests.


        :param service_id: Service ID of the resource to return
        :param unified_api: Unified API
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.VaultValidateConnectionStateRequest(
            service_id=service_id,
            unified_api=unified_api,
            request_body=utils.get_pydantic_model(
                request_body, Optional[models.VaultValidateConnectionStateRequestBody]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/vault/connections/{unified_api}/{service_id}/validate",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.VaultValidateConnectionStateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                True,
                "json",
                Optional[models.VaultValidateConnectionStateRequestBody],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="vault.validateConnectionState",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.VaultValidateConnectionStateResponse(
                validate_connection_state_response=utils.unmarshal_json(
                    http_res.text, Optional[models.ValidateConnectionStateResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.VaultValidateConnectionStateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def state_async(
        self,
        *,
        service_id: str,
        unified_api: str,
        request_body: Optional[
            Union[
                models.VaultValidateConnectionStateRequestBody,
                models.VaultValidateConnectionStateRequestBodyTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.VaultValidateConnectionStateResponse:
        r"""Validate Connection State

        This endpoint validates the current state of a given connection. This will perform different checks based on the connection auth type. For basic and apiKey auth types, the presence of required fields is checked.
        For connectors that implement OAuth2, this operation forces the refresh flow for an access token regardless of its expiry.

        Note:
        - Do not include any credentials in the request body. This operation does not persist changes, but only triggers the validation of connection state.
        - If a refresh token flow was performed and successful, the new access token will then be used for subsequent API requests.


        :param service_id: Service ID of the resource to return
        :param unified_api: Unified API
        :param request_body:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.VaultValidateConnectionStateRequest(
            service_id=service_id,
            unified_api=unified_api,
            request_body=utils.get_pydantic_model(
                request_body, Optional[models.VaultValidateConnectionStateRequestBody]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/vault/connections/{unified_api}/{service_id}/validate",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.VaultValidateConnectionStateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                True,
                "json",
                Optional[models.VaultValidateConnectionStateRequestBody],
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="vault.validateConnectionState",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.VaultValidateConnectionStateResponse(
                validate_connection_state_response=utils.unmarshal_json(
                    http_res.text, Optional[models.ValidateConnectionStateResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.VaultValidateConnectionStateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import Nullable, OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from datetime import datetime
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Expenses(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AccountingExpensesAllResponse]:
        r"""List Expenses

        List Expenses

        :param raw: Include raw response. Mostly used for debugging purposes
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesAllRequest(
            raw=raw,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/accounting/expenses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="accounting.expensesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AccountingExpensesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None
            next_cursor = next_cursor[0]

            return self.list(
                raw=raw,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                retries=retries,
            )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesAllResponse(
                get_expenses_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetExpensesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AccountingExpensesAllResponse]:
        r"""List Expenses

        List Expenses

        :param raw: Include raw response. Mostly used for debugging purposes
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesAllRequest(
            raw=raw,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounting/expenses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="accounting.expensesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AccountingExpensesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Dict[Any, Any])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None
            next_cursor = next_cursor[0]

            return self.list(
                raw=raw,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                retries=retries,
            )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesAllResponse(
                get_expenses_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetExpensesResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesAllResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def create(
        self,
        *,
        transaction_date: Nullable[datetime],
        account_id: str,
        line_items: Union[
            List[models.ExpenseLineItemInput],
            List[models.ExpenseLineItemInputTypedDict],
        ],
        raw: Optional[bool] = False,
        service_id: Optional[str] = None,
        number: OptionalNullable[str] = UNSET,
        customer_id: Optional[str] = None,
        supplier_id: Optional[str] = None,
        company_id: OptionalNullable[str] = UNSET,
        department_id: Optional[str] = None,
        payment_type: OptionalNullable[models.ExpensePaymentType] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        type_: OptionalNullable[models.ExpenseType] = UNSET,
        memo: OptionalNullable[str] = UNSET,
        tax_rate: Optional[
            Union[models.LinkedTaxRateInput, models.LinkedTaxRateInputTypedDict]
        ] = None,
        total_amount: OptionalNullable[float] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesAddResponse:
        r"""Create Expense

        Create Expense

        :param transaction_date: The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD
        :param account_id: The unique identifier for the ledger account that this expense should be credited to.
        :param line_items: Expense line items linked to this expense.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param number: Number.
        :param customer_id: The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers.
        :param supplier_id: The ID of the supplier this entity is linked to.
        :param company_id: The company or subsidiary id the transaction belongs to
        :param department_id: The ID of the department this expense is linked to.
        :param payment_type: The type of payment for the expense.
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param type: The type of expense.
        :param memo: The memo of the expense.
        :param tax_rate:
        :param total_amount: The total amount of the expense line item.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesAddRequest(
            raw=raw,
            service_id=service_id,
            expense=models.ExpenseInput(
                number=number,
                transaction_date=transaction_date,
                account_id=account_id,
                customer_id=customer_id,
                supplier_id=supplier_id,
                company_id=company_id,
                department_id=department_id,
                payment_type=payment_type,
                currency=currency,
                currency_rate=currency_rate,
                type=type_,
                memo=memo,
                tax_rate=utils.get_pydantic_model(
                    tax_rate, Optional[models.LinkedTaxRateInput]
                ),
                total_amount=total_amount,
                line_items=utils.get_pydantic_model(
                    line_items, List[models.ExpenseLineItemInput]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounting/expenses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.expense, False, False, "json", models.ExpenseInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="accounting.expensesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.AccountingExpensesAddResponse(
                create_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def create_async(
        self,
        *,
        transaction_date: Nullable[datetime],
        account_id: str,
        line_items: Union[
            List[models.ExpenseLineItemInput],
            List[models.ExpenseLineItemInputTypedDict],
        ],
        raw: Optional[bool] = False,
        service_id: Optional[str] = None,
        number: OptionalNullable[str] = UNSET,
        customer_id: Optional[str] = None,
        supplier_id: Optional[str] = None,
        company_id: OptionalNullable[str] = UNSET,
        department_id: Optional[str] = None,
        payment_type: OptionalNullable[models.ExpensePaymentType] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        type_: OptionalNullable[models.ExpenseType] = UNSET,
        memo: OptionalNullable[str] = UNSET,
        tax_rate: Optional[
            Union[models.LinkedTaxRateInput, models.LinkedTaxRateInputTypedDict]
        ] = None,
        total_amount: OptionalNullable[float] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesAddResponse:
        r"""Create Expense

        Create Expense

        :param transaction_date: The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD
        :param account_id: The unique identifier for the ledger account that this expense should be credited to.
        :param line_items: Expense line items linked to this expense.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param number: Number.
        :param customer_id: The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers.
        :param supplier_id: The ID of the supplier this entity is linked to.
        :param company_id: The company or subsidiary id the transaction belongs to
        :param department_id: The ID of the department this expense is linked to.
        :param payment_type: The type of payment for the expense.
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param type: The type of expense.
        :param memo: The memo of the expense.
        :param tax_rate:
        :param total_amount: The total amount of the expense line item.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesAddRequest(
            raw=raw,
            service_id=service_id,
            expense=models.ExpenseInput(
                number=number,
                transaction_date=transaction_date,
                account_id=account_id,
                customer_id=customer_id,
                supplier_id=supplier_id,
                company_id=company_id,
                department_id=department_id,
                payment_type=payment_type,
                currency=currency,
                currency_rate=currency_rate,
                type=type_,
                memo=memo,
                tax_rate=utils.get_pydantic_model(
                    tax_rate, Optional[models.LinkedTaxRateInput]
                ),
                total_amount=total_amount,
                line_items=utils.get_pydantic_model(
                    line_items, List[models.ExpenseLineItemInput]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounting/expenses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.expense, False, False, "json", models.ExpenseInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="accounting.expensesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.AccountingExpensesAddResponse(
                create_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.CreateExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesAddResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        id: str,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesOneResponse:
        r"""Get Expense

        Get Expense

        :param id: ID of the record you are acting upon.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesOneRequest(
            id=id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="GET",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="accounting.expensesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesOneResponse(
                get_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        id: str,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesOneResponse:
        r"""Get Expense

        Get Expense

        :param id: ID of the record you are acting upon.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesOneRequest(
            id=id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="accounting.expensesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesOneResponse(
                get_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.GetExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesOneResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def update(
        self,
        *,
        id: str,
        transaction_date: Nullable[datetime],
        account_id: str,
        line_items: Union[
            List[models.ExpenseLineItemInput],
            List[models.ExpenseLineItemInputTypedDict],
        ],
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        number: OptionalNullable[str] = UNSET,
        customer_id: Optional[str] = None,
        supplier_id: Optional[str] = None,
        company_id: OptionalNullable[str] = UNSET,
        department_id: Optional[str] = None,
        payment_type: OptionalNullable[models.ExpensePaymentType] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        type_: OptionalNullable[models.ExpenseType] = UNSET,
        memo: OptionalNullable[str] = UNSET,
        tax_rate: Optional[
            Union[models.LinkedTaxRateInput, models.LinkedTaxRateInputTypedDict]
        ] = None,
        total_amount: OptionalNullable[float] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesUpdateResponse:
        r"""Update Expense

        Update Expense

        :param id: ID of the record you are acting upon.
        :param transaction_date: The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD
        :param account_id: The unique identifier for the ledger account that this expense should be credited to.
        :param line_items: Expense line items linked to this expense.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param number: Number.
        :param customer_id: The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers.
        :param supplier_id: The ID of the supplier this entity is linked to.
        :param company_id: The company or subsidiary id the transaction belongs to
        :param department_id: The ID of the department this expense is linked to.
        :param payment_type: The type of payment for the expense.
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param type: The type of expense.
        :param memo: The memo of the expense.
        :param tax_rate:
        :param total_amount: The total amount of the expense line item.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesUpdateRequest(
            id=id,
            service_id=service_id,
            raw=raw,
            expense=models.ExpenseInput(
                number=number,
                transaction_date=transaction_date,
                account_id=account_id,
                customer_id=customer_id,
                supplier_id=supplier_id,
                company_id=company_id,
                department_id=department_id,
                payment_type=payment_type,
                currency=currency,
                currency_rate=currency_rate,
                type=type_,
                memo=memo,
                tax_rate=utils.get_pydantic_model(
                    tax_rate, Optional[models.LinkedTaxRateInput]
                ),
                total_amount=total_amount,
                line_items=utils.get_pydantic_model(
                    line_items, List[models.ExpenseLineItemInput]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.expense, False, False, "json", models.ExpenseInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="accounting.expensesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesUpdateResponse(
                update_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def update_async(
        self,
        *,
        id: str,
        transaction_date: Nullable[datetime],
        account_id: str,
        line_items: Union[
            List[models.ExpenseLineItemInput],
            List[models.ExpenseLineItemInputTypedDict],
        ],
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        number: OptionalNullable[str] = UNSET,
        customer_id: Optional[str] = None,
        supplier_id: Optional[str] = None,
        company_id: OptionalNullable[str] = UNSET,
        department_id: Optional[str] = None,
        payment_type: OptionalNullable[models.ExpensePaymentType] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        type_: OptionalNullable[models.ExpenseType] = UNSET,
        memo: OptionalNullable[str] = UNSET,
        tax_rate: Optional[
            Union[models.LinkedTaxRateInput, models.LinkedTaxRateInputTypedDict]
        ] = None,
        total_amount: OptionalNullable[float] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesUpdateResponse:
        r"""Update Expense

        Update Expense

        :param id: ID of the record you are acting upon.
        :param transaction_date: The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD
        :param account_id: The unique identifier for the ledger account that this expense should be credited to.
        :param line_items: Expense line items linked to this expense.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param number: Number.
        :param customer_id: The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers.
        :param supplier_id: The ID of the supplier this entity is linked to.
        :param company_id: The company or subsidiary id the transaction belongs to
        :param department_id: The ID of the department this expense is linked to.
        :param payment_type: The type of payment for the expense.
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param type: The type of expense.
        :param memo: The memo of the expense.
        :param tax_rate:
        :param total_amount: The total amount of the expense line item.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesUpdateRequest(
            id=id,
            service_id=service_id,
            raw=raw,
            expense=models.ExpenseInput(
                number=number,
                transaction_date=transaction_date,
                account_id=account_id,
                customer_id=customer_id,
                supplier_id=supplier_id,
                company_id=company_id,
                department_id=department_id,
                payment_type=payment_type,
                currency=currency,
                currency_rate=currency_rate,
                type=type_,
                memo=memo,
                tax_rate=utils.get_pydantic_model(
                    tax_rate, Optional[models.LinkedTaxRateInput]
                ),
                total_amount=total_amount,
                line_items=utils.get_pydantic_model(
                    line_items, List[models.ExpenseLineItemInput]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.expense, False, False, "json", models.ExpenseInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="accounting.expensesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesUpdateResponse(
                update_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UpdateExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesUpdateResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def delete(
        self,
        *,
        id: str,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesDeleteResponse:
        r"""Delete Expense

        Delete Expense

        :param id: ID of the record you are acting upon.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesDeleteRequest(
            id=id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="DELETE",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="accounting.expensesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesDeleteResponse(
                delete_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def delete_async(
        self,
        *,
        id: str,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingExpensesDeleteResponse:
        r"""Delete Expense

        Delete Expense

        :param id: ID of the record you are acting upon.
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = models.AccountingExpensesDeleteRequest(
            id=id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/accounting/expenses/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingExpensesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5XX"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="accounting.expensesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingExpensesDeleteResponse(
                delete_expense_response=utils.unmarshal_json(
                    http_res.text, Optional[models.DeleteExpenseResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.BadRequestResponseData)
            raise models.BadRequestResponse(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnauthorizedResponseData)
            raise models.UnauthorizedResponse(data=data)
        if utils.match_response(http_res, "402", "application/json"):
            data = utils.unmarshal_json(
                http_res.text, models.PaymentRequiredResponseData
            )
            raise models.PaymentRequiredResponse(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.NotFoundResponseData)
            raise models.NotFoundResponse(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, models.UnprocessableResponseData)
            raise models.UnprocessableResponse(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingExpensesDeleteResponse(
                unexpected_error_response=utils.unmarshal_json(
                    http_res.text, Optional[models.UnexpectedErrorResponse]
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .custommappings import CustomMappings, CustomMappingsTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class CollectionUserTypedDict(TypedDict):
    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    name: NotRequired[Nullable[str]]
    r"""Full name of the user"""
    first_name: NotRequired[Nullable[str]]
    r"""First name of the user"""
    last_name: NotRequired[Nullable[str]]
    r"""Last name of the user"""
    email: NotRequired[Nullable[str]]
    r"""Email address of the user"""
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class CollectionUser(BaseModel):
    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    name: OptionalNullable[str] = UNSET
    r"""Full name of the user"""

    first_name: OptionalNullable[str] = UNSET
    r"""First name of the user"""

    last_name: OptionalNullable[str] = UNSET
    r"""Last name of the user"""

    email: OptionalNullable[str] = UNSET
    r"""Email address of the user"""

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "photo_url",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "id",
            "name",
            "first_name",
            "last_name",
            "email",
            "photo_url",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhook import Webhook, WebhookTypedDict
from apideck_unify.types import BaseModel
from typing_extensions import TypedDict


class DeleteWebhookResponseTypedDict(TypedDict):
    r"""Webhooks"""

    status_code: int
    r"""HTTP Response Status Code"""
    status: str
    r"""HTTP Response Status"""
    data: WebhookTypedDict


class DeleteWebhookResponse(BaseModel):
    r"""Webhooks"""

    status_code: int
    r"""HTTP Response Status Code"""

    status: str
    r"""HTTP Response Status"""

    data: Webhook



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .connectionstate import ConnectionState
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ValidateConnectionStateResponseDataTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The unique identifier of the connection."""
    state: NotRequired[ConnectionState]
    r"""[Connection state flow](#section/Connection-state)"""


class ValidateConnectionStateResponseData(BaseModel):
    id: Optional[str] = None
    r"""The unique identifier of the connection."""

    state: Optional[ConnectionState] = None
    r"""[Connection state flow](#section/Connection-state)"""


class ValidateConnectionStateResponseTypedDict(TypedDict):
    r"""Connection access token refreshed"""

    status_code: int
    r"""HTTP Response Status Code"""
    status: str
    r"""HTTP Response Status"""
    data: ValidateConnectionStateResponseDataTypedDict


class ValidateConnectionStateResponse(BaseModel):
    r"""Connection access token refreshed"""

    status_code: int
    r"""HTTP Response Status Code"""

    status: str
    r"""HTTP Response Status"""

    data: ValidateConnectionStateResponseData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .gender import Gender
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class PersonInputTypedDict(TypedDict):
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    gender: NotRequired[Nullable[Gender]]
    r"""The gender represents the gender identity of a person."""
    initials: NotRequired[Nullable[str]]
    r"""Initials of the person"""
    birthday: NotRequired[Nullable[date]]
    r"""Date of birth"""
    deceased_on: NotRequired[Nullable[date]]
    r"""Date of death"""


class PersonInput(BaseModel):
    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    gender: OptionalNullable[Gender] = UNSET
    r"""The gender represents the gender identity of a person."""

    initials: OptionalNullable[str] = UNSET
    r"""Initials of the person"""

    birthday: OptionalNullable[date] = UNSET
    r"""Date of birth"""

    deceased_on: OptionalNullable[date] = UNSET
    r"""Date of death"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "first_name",
            "last_name",
            "middle_name",
            "gender",
            "initials",
            "birthday",
            "deceased_on",
        ]
        nullable_fields = [
            "first_name",
            "last_name",
            "middle_name",
            "gender",
            "initials",
            "birthday",
            "deceased_on",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activityattendee import (
    ActivityAttendee,
    ActivityAttendeeInput,
    ActivityAttendeeInputTypedDict,
    ActivityAttendeeTypedDict,
)
from .address import Address, AddressTypedDict
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ActivityType(str, Enum):
    r"""The type of the activity"""

    CALL = "call"
    MEETING = "meeting"
    EMAIL = "email"
    NOTE = "note"
    TASK = "task"
    DEADLINE = "deadline"
    SEND_LETTER = "send-letter"
    SEND_QUOTE = "send-quote"
    OTHER = "other"


class ShowAs(str, Enum):
    FREE = "free"
    BUSY = "busy"


class ActivityTypedDict(TypedDict):
    type: Nullable[ActivityType]
    r"""The type of the activity"""
    id: NotRequired[str]
    r"""The unique identifier of the activity"""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    activity_datetime: NotRequired[Nullable[str]]
    r"""The date and time of the activity"""
    duration_seconds: NotRequired[Nullable[int]]
    r"""The duration of the activity in seconds"""
    user_id: NotRequired[Nullable[str]]
    r"""The user related to the activity"""
    account_id: NotRequired[Nullable[str]]
    r"""The account related to the activity"""
    contact_id: NotRequired[Nullable[str]]
    r"""The contact related to the activity"""
    company_id: NotRequired[Nullable[str]]
    r"""The company related to the activity"""
    opportunity_id: NotRequired[Nullable[str]]
    r"""The opportunity related to the activity"""
    lead_id: NotRequired[Nullable[str]]
    r"""The lead related to the activity"""
    owner_id: NotRequired[Nullable[str]]
    r"""The owner of the activity"""
    campaign_id: NotRequired[Nullable[str]]
    r"""The campaign related to the activity"""
    case_id: NotRequired[Nullable[str]]
    r"""The case related to the activity"""
    asset_id: NotRequired[Nullable[str]]
    r"""The asset related to the activity"""
    contract_id: NotRequired[Nullable[str]]
    r"""The contract related to the activity"""
    product_id: NotRequired[Nullable[str]]
    r"""The product related to the activity"""
    solution_id: NotRequired[Nullable[str]]
    r"""The solution related to the activity"""
    custom_object_id: NotRequired[Nullable[str]]
    r"""The custom object related to the activity"""
    title: NotRequired[Nullable[str]]
    r"""The title of the activity"""
    description: NotRequired[Nullable[str]]
    r"""A description of the activity"""
    note: NotRequired[Nullable[str]]
    r"""An internal note about the activity"""
    location: NotRequired[Nullable[str]]
    r"""The location of the activity"""
    location_address: NotRequired[AddressTypedDict]
    all_day_event: NotRequired[Nullable[bool]]
    r"""Whether the Activity is an all day event or not"""
    private: NotRequired[Nullable[bool]]
    r"""Whether the Activity is private or not"""
    group_event: NotRequired[Nullable[bool]]
    r"""Whether the Activity is a group event or not"""
    event_sub_type: NotRequired[Nullable[str]]
    r"""The sub type of the group event"""
    group_event_type: NotRequired[Nullable[str]]
    r"""The type of the group event"""
    child: NotRequired[Nullable[bool]]
    r"""Whether the activity is a child of another activity or not"""
    archived: NotRequired[Nullable[bool]]
    r"""Whether the activity is archived or not"""
    deleted: NotRequired[Nullable[bool]]
    r"""Whether the activity is deleted or not"""
    show_as: NotRequired[Nullable[ShowAs]]
    done: NotRequired[Nullable[bool]]
    r"""Whether the Activity is done or not"""
    start_datetime: NotRequired[Nullable[str]]
    r"""The start date and time of the activity"""
    end_datetime: NotRequired[Nullable[str]]
    r"""The end date and time of the activity"""
    duration_minutes: NotRequired[Nullable[int]]
    r"""The duration of the activity in minutes"""
    activity_date: NotRequired[Nullable[str]]
    r"""The date of the activity"""
    end_date: NotRequired[Nullable[str]]
    r"""The end date of the activity"""
    recurrent: NotRequired[bool]
    r"""Whether the activity is recurrent or not"""
    reminder_datetime: NotRequired[Nullable[str]]
    r"""The date and time of the reminder"""
    reminder_set: NotRequired[Nullable[bool]]
    r"""Whether the reminder is set or not"""
    video_conference_url: NotRequired[Nullable[str]]
    r"""The URL of the video conference"""
    video_conference_id: NotRequired[Nullable[str]]
    r"""The ID of the video conference"""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    r"""Custom fields of the activity"""
    attendees: NotRequired[List[ActivityAttendeeTypedDict]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the activity"""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the activity"""
    updated_at: NotRequired[Nullable[str]]
    r"""The date and time when the activity was last updated"""
    created_at: NotRequired[Nullable[str]]
    r"""The date and time when the activity was created"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Activity(BaseModel):
    type: Nullable[ActivityType]
    r"""The type of the activity"""

    id: Optional[str] = None
    r"""The unique identifier of the activity"""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    activity_datetime: OptionalNullable[str] = UNSET
    r"""The date and time of the activity"""

    duration_seconds: OptionalNullable[int] = UNSET
    r"""The duration of the activity in seconds"""

    user_id: OptionalNullable[str] = UNSET
    r"""The user related to the activity"""

    account_id: OptionalNullable[str] = UNSET
    r"""The account related to the activity"""

    contact_id: OptionalNullable[str] = UNSET
    r"""The contact related to the activity"""

    company_id: OptionalNullable[str] = UNSET
    r"""The company related to the activity"""

    opportunity_id: OptionalNullable[str] = UNSET
    r"""The opportunity related to the activity"""

    lead_id: OptionalNullable[str] = UNSET
    r"""The lead related to the activity"""

    owner_id: OptionalNullable[str] = UNSET
    r"""The owner of the activity"""

    campaign_id: OptionalNullable[str] = UNSET
    r"""The campaign related to the activity"""

    case_id: OptionalNullable[str] = UNSET
    r"""The case related to the activity"""

    asset_id: OptionalNullable[str] = UNSET
    r"""The asset related to the activity"""

    contract_id: OptionalNullable[str] = UNSET
    r"""The contract related to the activity"""

    product_id: OptionalNullable[str] = UNSET
    r"""The product related to the activity"""

    solution_id: OptionalNullable[str] = UNSET
    r"""The solution related to the activity"""

    custom_object_id: OptionalNullable[str] = UNSET
    r"""The custom object related to the activity"""

    title: OptionalNullable[str] = UNSET
    r"""The title of the activity"""

    description: OptionalNullable[str] = UNSET
    r"""A description of the activity"""

    note: OptionalNullable[str] = UNSET
    r"""An internal note about the activity"""

    location: OptionalNullable[str] = UNSET
    r"""The location of the activity"""

    location_address: Optional[Address] = None

    all_day_event: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is an all day event or not"""

    private: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is private or not"""

    group_event: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is a group event or not"""

    event_sub_type: OptionalNullable[str] = UNSET
    r"""The sub type of the group event"""

    group_event_type: OptionalNullable[str] = UNSET
    r"""The type of the group event"""

    child: OptionalNullable[bool] = UNSET
    r"""Whether the activity is a child of another activity or not"""

    archived: OptionalNullable[bool] = UNSET
    r"""Whether the activity is archived or not"""

    deleted: OptionalNullable[bool] = UNSET
    r"""Whether the activity is deleted or not"""

    show_as: OptionalNullable[ShowAs] = UNSET

    done: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is done or not"""

    start_datetime: OptionalNullable[str] = UNSET
    r"""The start date and time of the activity"""

    end_datetime: OptionalNullable[str] = UNSET
    r"""The end date and time of the activity"""

    duration_minutes: OptionalNullable[int] = UNSET
    r"""The duration of the activity in minutes"""

    activity_date: OptionalNullable[str] = UNSET
    r"""The date of the activity"""

    end_date: OptionalNullable[str] = UNSET
    r"""The end date of the activity"""

    recurrent: Optional[bool] = None
    r"""Whether the activity is recurrent or not"""

    reminder_datetime: OptionalNullable[str] = UNSET
    r"""The date and time of the reminder"""

    reminder_set: OptionalNullable[bool] = UNSET
    r"""Whether the reminder is set or not"""

    video_conference_url: OptionalNullable[str] = UNSET
    r"""The URL of the video conference"""

    video_conference_id: OptionalNullable[str] = UNSET
    r"""The ID of the video conference"""

    custom_fields: Optional[List[CustomField]] = None
    r"""Custom fields of the activity"""

    attendees: Optional[List[ActivityAttendee]] = None

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the activity"""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the activity"""

    updated_at: OptionalNullable[str] = UNSET
    r"""The date and time when the activity was last updated"""

    created_at: OptionalNullable[str] = UNSET
    r"""The date and time when the activity was created"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "downstream_id",
            "activity_datetime",
            "duration_seconds",
            "user_id",
            "account_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "owner_id",
            "campaign_id",
            "case_id",
            "asset_id",
            "contract_id",
            "product_id",
            "solution_id",
            "custom_object_id",
            "title",
            "description",
            "note",
            "location",
            "location_address",
            "all_day_event",
            "private",
            "group_event",
            "event_sub_type",
            "group_event_type",
            "child",
            "archived",
            "deleted",
            "show_as",
            "done",
            "start_datetime",
            "end_datetime",
            "duration_minutes",
            "activity_date",
            "end_date",
            "recurrent",
            "reminder_datetime",
            "reminder_set",
            "video_conference_url",
            "video_conference_id",
            "custom_fields",
            "attendees",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "type",
            "downstream_id",
            "activity_datetime",
            "duration_seconds",
            "user_id",
            "account_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "owner_id",
            "campaign_id",
            "case_id",
            "asset_id",
            "contract_id",
            "product_id",
            "solution_id",
            "custom_object_id",
            "title",
            "description",
            "note",
            "location",
            "all_day_event",
            "private",
            "group_event",
            "event_sub_type",
            "group_event_type",
            "child",
            "archived",
            "deleted",
            "show_as",
            "done",
            "start_datetime",
            "end_datetime",
            "duration_minutes",
            "activity_date",
            "end_date",
            "reminder_datetime",
            "reminder_set",
            "video_conference_url",
            "video_conference_id",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ActivityInputTypedDict(TypedDict):
    type: Nullable[ActivityType]
    r"""The type of the activity"""
    activity_datetime: NotRequired[Nullable[str]]
    r"""The date and time of the activity"""
    duration_seconds: NotRequired[Nullable[int]]
    r"""The duration of the activity in seconds"""
    user_id: NotRequired[Nullable[str]]
    r"""The user related to the activity"""
    account_id: NotRequired[Nullable[str]]
    r"""The account related to the activity"""
    contact_id: NotRequired[Nullable[str]]
    r"""The contact related to the activity"""
    company_id: NotRequired[Nullable[str]]
    r"""The company related to the activity"""
    opportunity_id: NotRequired[Nullable[str]]
    r"""The opportunity related to the activity"""
    lead_id: NotRequired[Nullable[str]]
    r"""The lead related to the activity"""
    owner_id: NotRequired[Nullable[str]]
    r"""The owner of the activity"""
    campaign_id: NotRequired[Nullable[str]]
    r"""The campaign related to the activity"""
    case_id: NotRequired[Nullable[str]]
    r"""The case related to the activity"""
    asset_id: NotRequired[Nullable[str]]
    r"""The asset related to the activity"""
    contract_id: NotRequired[Nullable[str]]
    r"""The contract related to the activity"""
    product_id: NotRequired[Nullable[str]]
    r"""The product related to the activity"""
    solution_id: NotRequired[Nullable[str]]
    r"""The solution related to the activity"""
    custom_object_id: NotRequired[Nullable[str]]
    r"""The custom object related to the activity"""
    title: NotRequired[Nullable[str]]
    r"""The title of the activity"""
    description: NotRequired[Nullable[str]]
    r"""A description of the activity"""
    note: NotRequired[Nullable[str]]
    r"""An internal note about the activity"""
    location: NotRequired[Nullable[str]]
    r"""The location of the activity"""
    location_address: NotRequired[AddressTypedDict]
    all_day_event: NotRequired[Nullable[bool]]
    r"""Whether the Activity is an all day event or not"""
    private: NotRequired[Nullable[bool]]
    r"""Whether the Activity is private or not"""
    group_event: NotRequired[Nullable[bool]]
    r"""Whether the Activity is a group event or not"""
    event_sub_type: NotRequired[Nullable[str]]
    r"""The sub type of the group event"""
    group_event_type: NotRequired[Nullable[str]]
    r"""The type of the group event"""
    child: NotRequired[Nullable[bool]]
    r"""Whether the activity is a child of another activity or not"""
    archived: NotRequired[Nullable[bool]]
    r"""Whether the activity is archived or not"""
    deleted: NotRequired[Nullable[bool]]
    r"""Whether the activity is deleted or not"""
    show_as: NotRequired[Nullable[ShowAs]]
    done: NotRequired[Nullable[bool]]
    r"""Whether the Activity is done or not"""
    start_datetime: NotRequired[Nullable[str]]
    r"""The start date and time of the activity"""
    end_datetime: NotRequired[Nullable[str]]
    r"""The end date and time of the activity"""
    activity_date: NotRequired[Nullable[str]]
    r"""The date of the activity"""
    end_date: NotRequired[Nullable[str]]
    r"""The end date of the activity"""
    recurrent: NotRequired[bool]
    r"""Whether the activity is recurrent or not"""
    reminder_datetime: NotRequired[Nullable[str]]
    r"""The date and time of the reminder"""
    reminder_set: NotRequired[Nullable[bool]]
    r"""Whether the reminder is set or not"""
    video_conference_url: NotRequired[Nullable[str]]
    r"""The URL of the video conference"""
    video_conference_id: NotRequired[Nullable[str]]
    r"""The ID of the video conference"""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    r"""Custom fields of the activity"""
    attendees: NotRequired[List[ActivityAttendeeInputTypedDict]]
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class ActivityInput(BaseModel):
    type: Nullable[ActivityType]
    r"""The type of the activity"""

    activity_datetime: OptionalNullable[str] = UNSET
    r"""The date and time of the activity"""

    duration_seconds: OptionalNullable[int] = UNSET
    r"""The duration of the activity in seconds"""

    user_id: OptionalNullable[str] = UNSET
    r"""The user related to the activity"""

    account_id: OptionalNullable[str] = UNSET
    r"""The account related to the activity"""

    contact_id: OptionalNullable[str] = UNSET
    r"""The contact related to the activity"""

    company_id: OptionalNullable[str] = UNSET
    r"""The company related to the activity"""

    opportunity_id: OptionalNullable[str] = UNSET
    r"""The opportunity related to the activity"""

    lead_id: OptionalNullable[str] = UNSET
    r"""The lead related to the activity"""

    owner_id: OptionalNullable[str] = UNSET
    r"""The owner of the activity"""

    campaign_id: OptionalNullable[str] = UNSET
    r"""The campaign related to the activity"""

    case_id: OptionalNullable[str] = UNSET
    r"""The case related to the activity"""

    asset_id: OptionalNullable[str] = UNSET
    r"""The asset related to the activity"""

    contract_id: OptionalNullable[str] = UNSET
    r"""The contract related to the activity"""

    product_id: OptionalNullable[str] = UNSET
    r"""The product related to the activity"""

    solution_id: OptionalNullable[str] = UNSET
    r"""The solution related to the activity"""

    custom_object_id: OptionalNullable[str] = UNSET
    r"""The custom object related to the activity"""

    title: OptionalNullable[str] = UNSET
    r"""The title of the activity"""

    description: OptionalNullable[str] = UNSET
    r"""A description of the activity"""

    note: OptionalNullable[str] = UNSET
    r"""An internal note about the activity"""

    location: OptionalNullable[str] = UNSET
    r"""The location of the activity"""

    location_address: Optional[Address] = None

    all_day_event: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is an all day event or not"""

    private: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is private or not"""

    group_event: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is a group event or not"""

    event_sub_type: OptionalNullable[str] = UNSET
    r"""The sub type of the group event"""

    group_event_type: OptionalNullable[str] = UNSET
    r"""The type of the group event"""

    child: OptionalNullable[bool] = UNSET
    r"""Whether the activity is a child of another activity or not"""

    archived: OptionalNullable[bool] = UNSET
    r"""Whether the activity is archived or not"""

    deleted: OptionalNullable[bool] = UNSET
    r"""Whether the activity is deleted or not"""

    show_as: OptionalNullable[ShowAs] = UNSET

    done: OptionalNullable[bool] = UNSET
    r"""Whether the Activity is done or not"""

    start_datetime: OptionalNullable[str] = UNSET
    r"""The start date and time of the activity"""

    end_datetime: OptionalNullable[str] = UNSET
    r"""The end date and time of the activity"""

    activity_date: OptionalNullable[str] = UNSET
    r"""The date of the activity"""

    end_date: OptionalNullable[str] = UNSET
    r"""The end date of the activity"""

    recurrent: Optional[bool] = None
    r"""Whether the activity is recurrent or not"""

    reminder_datetime: OptionalNullable[str] = UNSET
    r"""The date and time of the reminder"""

    reminder_set: OptionalNullable[bool] = UNSET
    r"""Whether the reminder is set or not"""

    video_conference_url: OptionalNullable[str] = UNSET
    r"""The URL of the video conference"""

    video_conference_id: OptionalNullable[str] = UNSET
    r"""The ID of the video conference"""

    custom_fields: Optional[List[CustomField]] = None
    r"""Custom fields of the activity"""

    attendees: Optional[List[ActivityAttendeeInput]] = None

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "activity_datetime",
            "duration_seconds",
            "user_id",
            "account_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "owner_id",
            "campaign_id",
            "case_id",
            "asset_id",
            "contract_id",
            "product_id",
            "solution_id",
            "custom_object_id",
            "title",
            "description",
            "note",
            "location",
            "location_address",
            "all_day_event",
            "private",
            "group_event",
            "event_sub_type",
            "group_event_type",
            "child",
            "archived",
            "deleted",
            "show_as",
            "done",
            "start_datetime",
            "end_datetime",
            "activity_date",
            "end_date",
            "recurrent",
            "reminder_datetime",
            "reminder_set",
            "video_conference_url",
            "video_conference_id",
            "custom_fields",
            "attendees",
            "pass_through",
        ]
        nullable_fields = [
            "type",
            "activity_datetime",
            "duration_seconds",
            "user_id",
            "account_id",
            "contact_id",
            "company_id",
            "opportunity_id",
            "lead_id",
            "owner_id",
            "campaign_id",
            "case_id",
            "asset_id",
            "contract_id",
            "product_id",
            "solution_id",
            "custom_object_id",
            "title",
            "description",
            "note",
            "location",
            "all_day_event",
            "private",
            "group_event",
            "event_sub_type",
            "group_event_type",
            "child",
            "archived",
            "deleted",
            "show_as",
            "done",
            "start_datetime",
            "end_datetime",
            "activity_date",
            "end_date",
            "reminder_datetime",
            "reminder_set",
            "video_conference_url",
            "video_conference_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.drivegroups import DriveGroups
from apideck_unify.drives import Drives
from apideck_unify.files import Files
from apideck_unify.folders import Folders
from apideck_unify.sharedlinks import SharedLinks
from apideck_unify.uploadsessions import UploadSessions


class FileStorage(BaseSDK):
    files: Files
    folders: Folders
    shared_links: SharedLinks
    upload_sessions: UploadSessions
    drives: Drives
    drive_groups: DriveGroups

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.files = Files(self.sdk_configuration)
        self.folders = Folders(self.sdk_configuration)
        self.shared_links = SharedLinks(self.sdk_configuration)
        self.upload_sessions = UploadSessions(self.sdk_configuration)
        self.drives = Drives(self.sdk_configuration)
        self.drive_groups = DriveGroups(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .custommappings import CustomMappings, CustomMappingsTypedDict
from .email import Email, EmailTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from .sociallink import SocialLink, SocialLinkTypedDict
from .website import Website, WebsiteTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class LeadTypedDict(TypedDict):
    name: str
    r"""Full name of the lead."""
    company_name: Nullable[str]
    r"""The name of the company the lead is associated with."""
    id: NotRequired[str]
    r"""Unique identifier for the contact."""
    owner_id: NotRequired[Nullable[str]]
    r"""The owner of the lead."""
    owner_name: NotRequired[Nullable[str]]
    r"""The name of the owner of the lead."""
    company_id: NotRequired[Nullable[str]]
    r"""The company the lead is associated with."""
    lead_id: NotRequired[Nullable[str]]
    r"""The identifier of the lead."""
    lead_source: NotRequired[Nullable[str]]
    r"""The source of the lead."""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the lead."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the lead."""
    description: NotRequired[Nullable[str]]
    r"""The description of the lead."""
    prefix: NotRequired[Nullable[str]]
    r"""The prefix of the lead."""
    title: NotRequired[Nullable[str]]
    r"""The job title of the lead."""
    language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    status: NotRequired[Nullable[str]]
    monetary_amount: NotRequired[Nullable[float]]
    r"""The monetary amount of the lead."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    fax: NotRequired[Nullable[str]]
    r"""The fax number of the lead."""
    websites: NotRequired[List[WebsiteTypedDict]]
    addresses: NotRequired[List[AddressTypedDict]]
    social_links: NotRequired[List[SocialLinkTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    tags: NotRequired[Nullable[List[str]]]
    custom_mappings: NotRequired[Nullable[CustomMappingsTypedDict]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_at: NotRequired[Nullable[str]]
    r"""Date updated in ISO 8601 format"""
    created_at: NotRequired[Nullable[str]]
    r"""Date created in ISO 8601 format"""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Lead(BaseModel):
    name: str
    r"""Full name of the lead."""

    company_name: Nullable[str]
    r"""The name of the company the lead is associated with."""

    id: Optional[str] = None
    r"""Unique identifier for the contact."""

    owner_id: OptionalNullable[str] = UNSET
    r"""The owner of the lead."""

    owner_name: OptionalNullable[str] = UNSET
    r"""The name of the owner of the lead."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company the lead is associated with."""

    lead_id: OptionalNullable[str] = UNSET
    r"""The identifier of the lead."""

    lead_source: OptionalNullable[str] = UNSET
    r"""The source of the lead."""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the lead."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the lead."""

    description: OptionalNullable[str] = UNSET
    r"""The description of the lead."""

    prefix: OptionalNullable[str] = UNSET
    r"""The prefix of the lead."""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the lead."""

    language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    status: OptionalNullable[str] = UNSET

    monetary_amount: OptionalNullable[float] = UNSET
    r"""The monetary amount of the lead."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    fax: OptionalNullable[str] = UNSET
    r"""The fax number of the lead."""

    websites: Optional[List[Website]] = None

    addresses: Optional[List[Address]] = None

    social_links: Optional[List[SocialLink]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    tags: OptionalNullable[List[str]] = UNSET

    custom_mappings: OptionalNullable[CustomMappings] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_at: OptionalNullable[str] = UNSET
    r"""Date updated in ISO 8601 format"""

    created_at: OptionalNullable[str] = UNSET
    r"""Date created in ISO 8601 format"""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "owner_id",
            "owner_name",
            "company_id",
            "lead_id",
            "lead_source",
            "first_name",
            "last_name",
            "description",
            "prefix",
            "title",
            "language",
            "status",
            "monetary_amount",
            "currency",
            "fax",
            "websites",
            "addresses",
            "social_links",
            "phone_numbers",
            "emails",
            "custom_fields",
            "tags",
            "custom_mappings",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "company_name",
            "owner_id",
            "owner_name",
            "company_id",
            "lead_id",
            "lead_source",
            "first_name",
            "last_name",
            "description",
            "prefix",
            "title",
            "language",
            "status",
            "monetary_amount",
            "currency",
            "fax",
            "tags",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .balancebytransaction import BalanceByTransaction, BalanceByTransactionTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class BalanceByPeriodTypedDict(TypedDict):
    start_date: NotRequired[Nullable[date]]
    r"""The starting date of the period. If not provided, it represents the oldest period, where all transactions due before the specified `end_date` are included."""
    end_date: NotRequired[Nullable[date]]
    r"""The ending date of the period. If not provided, it represents an open-ended period starting from the `start_date`, typically capturing future-dated transactions that are not yet aged."""
    total_amount: NotRequired[float]
    r"""Total amount of the period."""
    balances_by_transaction: NotRequired[List[BalanceByTransactionTypedDict]]


class BalanceByPeriod(BaseModel):
    start_date: OptionalNullable[date] = UNSET
    r"""The starting date of the period. If not provided, it represents the oldest period, where all transactions due before the specified `end_date` are included."""

    end_date: OptionalNullable[date] = UNSET
    r"""The ending date of the period. If not provided, it represents an open-ended period starting from the `start_date`, typically capturing future-dated transactions that are not yet aged."""

    total_amount: Optional[float] = None
    r"""Total amount of the period."""

    balances_by_transaction: Optional[List[BalanceByTransaction]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "start_date",
            "end_date",
            "total_amount",
            "balances_by_transaction",
        ]
        nullable_fields = ["start_date", "end_date"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
